local AceGUI = LibStub("AceGUI-3.0")

local tconcat, tremove, tinsert = table.concat, table.remove, table.insert

local function CreateDispatcher(argCount)
   local code = [[
        local xpcall, eh = ...
        local method, ARGS
        local function call() return method(ARGS) end
    
        local function dispatch(func, ...)
            method = func
            if not method then return end
            ARGS = ...
            return xpcall(call, eh)
        end
    
        return dispatch
    ]]
   
   local ARGS = {}
   for i = 1, argCount do ARGS[i] = "arg"..i end
   code = code:gsub("ARGS", tconcat(ARGS, ", "))
   return assert(loadstring(code, "safecall Dispatcher["..argCount.."]"))(xpcall, errorhandler)
end

local Dispatchers = setmetatable({}, {__index=function(self, argCount)
         local dispatcher = CreateDispatcher(argCount)
         rawset(self, argCount, dispatcher)
         return dispatcher
end})
Dispatchers[0] = function(func)
   return xpcall(func, errorhandler)
end

local function safecall(func, ...)
   return Dispatchers[select("#", ...)](func, ...)
end

AceGUI:RegisterLayout("TightList", function(content, children)
	
	local height = 0
	for key, child in pairs(children) do
		child:SetWidth(content:GetWidth())
		if key == 1 then
			child:SetPoint("TOPLEFT", content, -5, 5)
			child:SetPoint("RIGHT", content, 5, 0)
		else
			child:SetPoint("TOPLEFT", children[key - 1].frame, "BOTTOMLEFT", 0, 5)
			child:SetPoint("RIGHT", children[key - 1].frame)
		end
		height = height + child.frame:GetHeight() - 5
	end

    safecall(content.obj.LayoutFinished, content.obj, nil, height)
end)